import axios from 'axios'
import { ElNotification, ElMessageBox, ElMessage, ElLoading } from 'element-plus'
import { getToken, getTenant } from '@/utils/auth'
import errorCode from '@/utils/errorCode'
import { tansParams, blobValidate } from '@/utils/ruoyi'
import cache from '@/plugins/cache'
import { saveAs } from 'file-saver'
import useUserStore from '@/store/modules/user'
import Notification from '../plugins/notification'
// import emitter from '../eventBus'
// import { getCurrentInstance } from 'vue';
// import { reject } from 'lodash-es'

let downloadLoadingInstance
// 是否显示重新登录
export let isRelogin = { show: false }

axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8'

const preService = axios.create({
  baseURL: '',
  timeout: 1000,
})

// 创建axios实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  // 因为对应了多个后端服务，这里不要统一定义前缀
  baseURL: '', //import.meta.env.VITE_APP_BASE_API
  // 超时
  timeout: 1000000,
})

// request拦截器
service.interceptors.request.use(
  async (config) => {
    // 是否需要设置 token
    const isToken = (config.headers || {}).isToken === false
    // 是否需要防止数据重复提交
    const isRepeatSubmit = (config.headers || {}).repeatSubmit === false
    if (getToken() && !isToken) {
      config.headers['Authorization'] = 'Bearer ' + getToken() // 让每个请求携带自定义token 请根据实际情况自行修改
    }
    const tenantInfo = getTenant()
    if (tenantInfo) {
      config.headers['tenantId'] = tenantInfo.tenantId
    }
    // appKey 根据该值，获取菜单，不要修改
    config.headers['appKey'] = 'ezhizao-identity'
    let requestable = true
    let notifications = []
    // 当登录状态为已登录时才可以获取公告
    // console.log(config)
    if (getToken() && !isToken && tenantInfo && config.url != '/ezhizao-dms-sys/getInfo' && config.url != '/ezhizao-dms-sys/getRouters') {
      // 所有条件都具备的情况下需先请求公告
      // let url = import.meta.env.VITE_APP_BASE_API + '/business/notification/getUnshowedNotification'
      // const res = await axios({
      //   url: url,
      //   method: 'get',
      //   headers: {
      //     Authorization: 'Bearer ' + getToken(),
      //     tenantId: getTenant().tenantId,
      //     appKey: 'ezhizao-identity'
      //   }
      // })
      // if (res.data.code == 200) {

      // } else if (res.data.code == 202) {
      //   notifications = res.data.data
      //   requestable = false
      // }
    }
    // get请求映射params参数
    if (config.method === 'get' && config.params) {
      let url = config.url + '?' + tansParams(config.params)
      url = url.slice(0, -1)
      config.params = {}
      config.url = url
    }
    if (!isRepeatSubmit && (config.method === 'post' || config.method === 'put')) {
      const requestObj = {
        url: config.url,
        data: typeof config.data === 'object' ? JSON.stringify(config.data) : config.data,
        time: new Date().getTime()
      }
      const sessionObj = cache.session.getJSON('sessionObj')
      if (sessionObj === undefined || sessionObj === null || sessionObj === '') {
        cache.session.setJSON('sessionObj', requestObj)
      } else {
        const s_url = sessionObj.url // 请求地址
        const s_data = sessionObj.data // 请求数据
        const s_time = sessionObj.time // 请求时间
        const interval = 1000 // 间隔时间(ms)，小于此时间视为重复提交
        if (s_data === requestObj.data && requestObj.time - s_time < interval && s_url === requestObj.url) {
          const message = '数据正在处理，请勿重复提交'
          console.warn(`[${s_url}]: ` + message)
          return Promise.reject(new Error(message))
        } else {
          cache.session.setJSON('sessionObj', requestObj)
        }
      }
    }
    return requestable ? config : Promise.reject({ message: 'SHOW NOTIFICATION', data: notifications })
  },
  (error) => {
    console.log(error)
    Promise.reject(error)
  }
)

// 响应拦截器
service.interceptors.response.use(
  (res) => {
    // 未设置状态码则默认成功状态
    const code = res.data.code || 200
    // 获取错误信息
    const msg = errorCode[code] || res.data.msg || errorCode['default']
    // 二进制数据则直接返回
    if (res.request.responseType === 'blob' || res.request.responseType === 'arraybuffer') {
      return res.data
    }
    if (code === 401) {
      if (!isRelogin.show) {
        isRelogin.show = true
        ElMessageBox.confirm('登录状态已过期，您可以继续留在该页面，或者重新登录', '系统提示', {
          confirmButtonText: '重新登录',
          cancelButtonText: '取消',
          type: 'warning'
        })
          .then(() => {
            isRelogin.show = false
            useUserStore()
              .logOut()
              .then(() => {
                location.href = '/index'
              })
          })
          .catch(() => {
            isRelogin.show = false
          })
      }
      return Promise.reject('无效的会话，或者会话已过期，请重新登录。')
    } else if (code === 500) {
      ElMessage({ message: msg, type: 'error' })
      return Promise.reject(new Error(msg))
    } else if (code === 601) {
      ElMessage({ message: msg, type: 'warning' })
      return Promise.reject(new Error(msg))
    } else if (code !== 200) {
      ElNotification.error({ title: msg })
      return Promise.reject('error')
    } else {
      return Promise.resolve(res.data)
    }
  },
  (error) => {
    // console.log(error)
    // console.log('err' + error)
    let { message, data } = error
    if (message == 'Network Error') {
      message = '后端接口连接异常'
      ElMessage({ message: message, type: 'error', duration: 5 * 1000 })
    } else if (message.includes('timeout')) {
      message = '数据过多查询超时'
      ElMessage({ message: message, type: 'error', duration: 5 * 1000 })
    } else if (message.includes('Request failed with status code')) {
      message = '系统接口' + message.substr(message.length - 3) + '异常'
      ElMessage({ message: message, type: 'error', duration: 5 * 1000 })
    } else if (message.includes('SHOW NOTIFICATION')) {
      // showDialog(data)
      // Notification(data[0])
    } else {
      ElMessage({ message: message, type: 'error', duration: 5 * 1000 })
    }
    return Promise.reject(error)
  }
)

// 通用下载方法
export function download(url, params, filename, config) {
  downloadLoadingInstance = ElLoading.service({ text: '正在下载数据，请稍候', background: 'rgba(0, 0, 0, 0.7)' })
  return service
    .post(url, params, {
      transformRequest: [
        (params) => {
          return tansParams(params)
        }
      ],
      headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
      responseType: 'blob',
      ...config
    })
    .then(async (data) => {
      const isLogin = await blobValidate(data)
      if (isLogin) {
        const blob = new Blob([data])
        saveAs(blob, filename)
      } else {
        const resText = await data.text()
        const rspObj = JSON.parse(resText)
        const errMsg = errorCode[rspObj.code] || rspObj.msg || errorCode['default']
        ElMessage.error(errMsg)
      }
      downloadLoadingInstance.close()
    })
    .catch((r) => {
      console.error(r)
      ElMessage.error('下载文件出现错误，请联系管理员！')
      downloadLoadingInstance.close()
    })
}

// 通用pdf打印方法
export function downloadPdf(url, params, filename, config) {
  downloadLoadingInstance = ElLoading.service({ text: "正在下载数据，请稍候", background: "rgba(0, 0, 0, 0.7)", })
  return service.post(url, params, {
    transformRequest: [(params) => { return tansParams(params) }],
    headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
    responseType: 'blob',
    ...config
  }).then(async (data) => {
    const isLogin = await blobValidate(data);
    if (isLogin) {
      const blob = new Blob([data], { type: 'application/pdf' })
      var url = URL.createObjectURL(blob);
      window.open(url, 'filename');
    } else {
      const resText = await data.text();
      const rspObj = JSON.parse(resText);
      const errMsg = errorCode[rspObj.code] || rspObj.msg || errorCode['default']
      ElMessage.error(errMsg);
      throw new Error(errMsg);
    }
    downloadLoadingInstance.close();
  }).catch((r) => {
    console.error(r)
    ElMessage.error('下载文件出现错误，请联系管理员！')
    downloadLoadingInstance.close();
    throw new Error(r);
  })
}

function showDialog(message) {
  // console.log(message, emitter)
  // emitter.emit('show-dialog', message)
  console.log(emitter)
  // const instance = getCurrentInstance();
  // console.log(instance)
  // if (instance && instance.proxy && instance.proxy.showNotification) {
  //   instance.proxy.showNotification(message);
  // } else {
  //   console.error('Cannot show dialog: current instance is null or $showDialog is not available.');
  // }
}

export default service
